// SJPEGFile.cpp: Implementierung der Klasse SJPEGFile.
//
// Copyright by Andr Stein
// E-Mail: andre_stein@gmx.de, andre_stein@web.de
// http://stonemaster.port5.com
//////////////////////////////////////////////////////////////////////

#include "SJPEGFile.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SJPEGFile::SJPEGFile()
{
	imageData = NULL;
	imageWidth = 0;
	imageHeight = 0;
	imageChannels  = 0;
}

SJPEGFile::~SJPEGFile()
{
	cleanUp();
}

SJPEGFile::SJPEGFile(const char *filename,bool topDownBuffer)
{
	if (!loadJpegFile(filename,topDownBuffer))
	{
		CString error = CString(filename)+" konnte nicht geladen werden!";
		
		throw SJPEGError(error);
	}
}

//**************
//Funktionen
//**************

bool SJPEGFile::loadJpegFile(const char *filename,bool topDownBuffer)
{
	//***************
	//Subern
	//***************
	
	cleanUp();
	
	//***************
	//Alte Daten wurden gelscht und 
	//neu initalisiert
	//***************

	bool status;
	IJLERR error;

	int dibLineWidth;
	int dibPadBytes;
	int sizeOfImage;

	JPEG_CORE_PROPERTIES jpegProps;

	status = true;

	__try
	{
		error = ijlInit(&jpegProps);

		if (IJL_OK != error)
		{
			status = false;
			__leave;
		}

		jpegProps.JPGFile = (char*)filename;

		error = ijlRead(&jpegProps,IJL_JFILE_READPARAMS);

		if (IJL_OK != error)
		{
			status = false;
			__leave;
		}

		imageWidth = jpegProps.JPGWidth;
		imageHeight= jpegProps.JPGHeight;
		imageChannels = 3;

		dibLineWidth = imageWidth*imageChannels;
		dibPadBytes = IJL_DIB_PAD_BYTES(
			imageWidth,
			imageChannels);
		
		sizeOfImage = (dibLineWidth+dibPadBytes)*
					imageHeight;

		imageData = new BYTE[sizeOfImage];

		if (imageData == NULL)
		{
			status = false;
			__leave;
		}
		
		jpegProps.DIBWidth = imageWidth;

		//****************
		//Je nach Wert topDownBuffer
		//Bild von oben nach unten oder von unten nach oben
		//****************
		if (topDownBuffer)
		{
			jpegProps.DIBHeight = -imageHeight;
		}
		else
		{
			jpegProps.DIBHeight = imageHeight;
		}

		jpegProps.DIBChannels = imageChannels;
		jpegProps.DIBColor = IJL_BGR;
		jpegProps.DIBPadBytes = dibPadBytes;
		jpegProps.DIBBytes = imageData;

		switch (jpegProps.JPGChannels)
		{
		case 1:
			{
				jpegProps.JPGColor = IJL_G;
				break;
			}
		case 3:
			{
				jpegProps.JPGColor = IJL_YCBCR;
				break;
			}
		default:
			{
				jpegProps.DIBColor = (IJL_COLOR)IJL_OTHER;
				jpegProps.JPGColor = (IJL_COLOR)IJL_OTHER;
				break;
			}
		}
			
		error = ijlRead(&jpegProps,IJL_JFILE_READWHOLEIMAGE);

		if (IJL_OK != error)
		{
			status = false;
			__leave;
		}
	}
	__finally
	{
		if (!status)
		{
			if (imageData != NULL)
			{
				delete[] imageData;
				imageData = NULL;
			}
		}

		ijlFree(&jpegProps);
	}

	return status;
}

void SJPEGFile::cleanUp()
{
	//*************************
	//Alles vorherige wird gelscht
	//*************************

	if (imageData)
	{
		delete[] imageData;
		
		imageWidth = 0;
		imageHeight = 0;
		imageChannels = 0;
	}
}

